@@ -18,10 +18,10 @@
 from http.server import HTTPServer, BaseHTTPRequestHandler
 from jinja2 import __version__ as jinja2_version
 import jinjafx, os, io, sys, socket, signal, threading, yaml, json, base64, time, datetime, resource
-import re, argparse, zipfile, hashlib, traceback, glob, hmac, uuid, struct, binascii, gzip, requests
-import cmarkgfm, emoji, func_timeout
+import re, argparse, zipfile, hashlib, traceback, glob, hmac, uuid, struct, binascii, gzip, requests, ctypes
+import cmarkgfm, emoji
 
-__version__ = '22.12.2'
+__version__ = '23.2.0'
 
 lock = threading.RLock()
 base = os.path.abspath(os.path.dirname(__file__))
@@ -39,6 +39,8 @@ rl_rate = 0
 rl_limit = 0
 logfile = None
 timelimit = 0
+n_threads = 4
+
 
 class JinjaFxServer(HTTPServer):
   def handle_error(self, request, client_address):
@@ -69,7 +71,9 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
 
     if not self.hide or verbose:
       if not isinstance(args[0], int) and path != '/ping':
-        if args[1] == '200' or args[1] == '204':
+        if self.error is not None:
+          ansi = '31'
+        elif args[1] == '200' or args[1] == '204':
           ansi = '32'
         elif args[1] == '304':
           ansi = '33'
@@ -90,14 +94,16 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
               else:
                 ctype = ' (' + self.headers['Content-Type'] + ')'
 
-          if str(args[1]) == 'ERR':
-            log('[' + src + '] [\033[1;' + ansi + 'm' + str(args[1]) + '\033[0m]' + ' \033[1;' + ansi + 'm' + str(args[2]) + '\033[0m')
-        
-          elif self.command == 'POST':
-            if self.elapsed is not None:
-              log('[' + src + '] [\033[1;' + ansi + 'm' + str(args[1]) + '\033[0m]' + ' \033[1;33m' + self.command + '\033[0m ' + path + ctype + ' [' + self.format_bytes(self.length) + '] in ' + str(self.elapsed) + 'ms')
+          if self.command == 'POST':
+            if self.error is not None:
+              ae = ' ->\033[1;' + ansi + 'm ' + str(self.error)[5:] + '\033[0m'
             else:
-              log('[' + src + '] [\033[1;' + ansi + 'm' + str(args[1]) + '\033[0m]' + ' \033[1;33m' + self.command + '\033[0m ' + path + ctype + ' [' + self.format_bytes(self.length) + ']')
+              ae = ''
+
+            if self.elapsed is not None:
+              log('[' + src + '] [\033[1;' + ansi + 'm' + str(args[1]) + '\033[0m]' + ' \033[1;33m' + self.command + '\033[0m ' + path + ctype + ' [' + self.format_bytes(self.length) + '] in ' + str(self.elapsed) + 'ms', ae)
+            else:
+              log('[' + src + '] [\033[1;' + ansi + 'm' + str(args[1]) + '\033[0m]' + ' \033[1;33m' + self.command + '\033[0m ' + path + ctype + ' [' + self.format_bytes(self.length) + ']', ae)
 
           elif self.command != None:
             if (args[1] != '200' and args[1] != '304') or (not path.endswith('.js') and not path.endswith('.css') and not path.endswith('.png')) or verbose:
@@ -130,6 +136,7 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
     try:
       self.critical = False
       self.hide = False
+      self.error = None
 
       fpath = self.path.split('?', 1)[0]
   
@@ -289,18 +296,20 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
         self.wfile.write(r[2])
 
     except Exception as e:
-      log('Exception: ' + str(e))
+      log(traceback.format_exc())
 
 
   def do_OPTIONS(self):
     self.critical = False
     self.hide = False
+    self.error = None
     self.send_response(204)
     self.send_header('Allow', 'OPTIONS, HEAD, GET, POST')
     self.end_headers()
 
 
   def do_HEAD(self):
+    self.error = None
     self.do_GET(True)
 
 
@@ -308,6 +317,7 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
     self.critical = False
     self.hide = False
     self.elapsed = None
+    self.error = None
 
     uc = self.path.split('?', 1)
     params = { x[0]: x[1] for x in [x.split('=') for x in uc[1].split('&') ] } if len(uc) > 1 else { }
@@ -350,16 +360,28 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
                 if y != None:
                   gvars.update(y)
 
-              try:
-                st = round(time.time() * 1000)
-                ocount = 0
+              st = round(time.time() * 1000)
+              ocount = 0
+              ret = [0, None]
+              
+              t = StoppableJinjaFx(jinjafx.JinjaFx().jinjafx, template.decode('utf-8'), data.decode('utf-8'), gvars, ret)
 
-                if timelimit > 0:
-                  outputs = func_timeout.func_timeout(timelimit, jinjafx.JinjaFx().jinjafx, args=(template.decode('utf-8'), data.decode('utf-8'), gvars, 'Output', [], True))
-                else:
-                  outputs = jinjafx.JinjaFx().jinjafx(template.decode('utf-8'), data.decode('utf-8'), gvars, 'Output', [], True)
-                            
-              except func_timeout.FunctionTimedOut:
+              if timelimit > 0:
+                while t.is_alive() and ((time.time() * 1000) - st) <= (timelimit * 1000):
+                  time.sleep(0.1)
+
+                if t.is_alive():
+                  t.stop()
+
+              t.join()
+
+              if ret[0] == 1:
+                outputs = ret[1]
+
+              elif ret[0] == -1:
+                raise ret[1]
+
+              else:
                 raise Exception("execution time limit of " + str(timelimit) + "s exceeded")
 
               jsr = {
@@ -407,13 +429,15 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
               elif 'yaml.SafeLoader' in tb:
                 error = 'error[vars.yml]: ' + type(e).__name__ + ': ' + str(e)
               else:
-                error = 'error[' + str(sys.exc_info()[2].tb_lineno) + ']: ' + type(e).__name__ + ': ' + str(e)
+                exc_type, exc_obj, exc_tb = sys.exc_info()
+                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
+                error = 'error[jinjafx_server.py:' + str(exc_tb.tb_lineno) + ']: ' + type(e).__name__ + ': ' + str(e)
 
               jsr = {
                 'status': 'error',
                 'error': error
               }
-              self.log_request('ERR', error);
+              self.error = error
   
             r = [ 'application/json', 200, json.dumps(jsr), sys._getframe().f_lineno ]
   
@@ -456,7 +480,7 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
                 return
 
               except Exception as e:
-                log('Exception: ' + str(e))
+                log(traceback.format_exc())
                 r = [ 'text/plain', 400, '400 Bad Request\r\n', sys._getframe().f_lineno ]
 
             else:
@@ -503,6 +527,15 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
                     dt_yml += 'dt:\n'
 
                     if 'datasets' in dt:
+                      if 'global' in dt:
+                        vdt['global'] = base64.b64decode(dt['global']).decode('utf-8') if 'global' in dt and len(dt['global'].strip()) > 0 else ''
+
+                        if vdt['global'] == '':
+                          dt_yml += '  global: ""\n\n'
+                        else:
+                          dt_yml += '  global: |2\n'
+                          dt_yml += re.sub('^', ' ' * 4, vdt['global'].rstrip(), flags=re.MULTILINE) + '\n\n'
+
                       dt_yml += '  datasets:\n'
 
                       for ds in dt['datasets']:
@@ -681,7 +714,7 @@ class JinjaFxRequest(BaseHTTPRequestHandler):
                     r = [ 'text/plain', 429, '429 Too Many Requests\r\n', sys._getframe().f_lineno ]
 
                 except Exception as e:
-                  log('Exception: ' + str(e))
+                  log(traceback.format_exc())
                   r = [ 'text/plain', 400, '400 Bad Request\r\n', sys._getframe().f_lineno ]
 
               else:
@@ -726,7 +759,6 @@ class JinjaFxThread(threading.Thread):
     self.daemon = True
     self.start()
 
-
   def run(self):
     httpd = JinjaFxServer(self.addr, JinjaFxRequest, False)
     httpd.socket = self.s
@@ -734,6 +766,29 @@ class JinjaFxThread(threading.Thread):
     httpd.serve_forever()
 
 
+class StoppableJinjaFx(threading.Thread):
+  def __init__(self, jinjafx, template, data, gvars, ret):
+    threading.Thread.__init__(self)
+    self.jinjafx = jinjafx
+    self.template = template
+    self.data = data
+    self.gvars = gvars
+    self.ret = ret
+    self.start()
+
+  def run(self):
+    try:
+      self.ret[1] = self.jinjafx(self.template, self.data, self.gvars, 'Output', [], True)
+      self.ret[0] = 1
+
+    except Exception as e:
+      self.ret[1] = e
+      self.ret[0] = -1
+
+  def stop(self):
+    ctypes.pythonapi.PyThreadState_SetAsyncExc(ctypes.c_long(self.ident), ctypes.py_object(SystemExit))
+
+
 def main(rflag=[0]):
   global aws_s3_url
   global aws_access_key
@@ -811,18 +866,20 @@ def main(rflag=[0]):
       soft, hard = resource.getrlimit(resource.RLIMIT_AS)
       resource.setrlimit(resource.RLIMIT_AS, (args.ml * 1024 * 1024, hard))
 
+    update_versioned_links(base + '/www')
+
     log('Starting JinjaFx Server (PID is ' + str(os.getpid()) + ') on http://' + args.l + ':' + str(args.p) + '...')
 
     s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
     s.bind((args.l, args.p))
-    s.listen(16)
+    s.listen()
 
     rflag[0] = 1
     threads = []
     repository = args.r
 
-    for i in range(16):
+    for i in range(n_threads):
       threads.append(JinjaFxThread(s, (args.l, args.p)))
 
     while rflag[0] < 2:
@@ -833,7 +890,8 @@ def main(rflag=[0]):
 
   except Exception as e:
     exc_type, exc_obj, exc_tb = sys.exc_info()
-    print('error[' + str(exc_tb.tb_lineno) + ']: ' + str(e), file=sys.stderr)
+    fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
+    print('error[jinjafx_server.py:' + str(exc_tb.tb_lineno) + ']: ' + str(e), file=sys.stderr)
     sys.exit(-2)
 
   finally:
@@ -842,21 +900,45 @@ def main(rflag=[0]):
       s.close()
 
 
-def log(t):
+def log(t, ae=''):
   with lock:
     timestamp = datetime.datetime.now().strftime('%b %d %H:%M:%S.%f')[:19]
-    print('[' + timestamp + '] ' + t)
+    print('[' + timestamp + '] ' + t + ae)
 
     if logfile is not None:
       try:
         with open(logfile, 'at') as f:
-          f.write('[' + timestamp + '] ' + re.sub(r'\033\[(?:1;[0-9][0-9]|0)m', '', t) + '\n')
+          f.write('[' + timestamp + '] ' + re.sub(r'\033\[(?:1;[0-9][0-9]|0)m', '', t + ae) + '\n')
 
       except Exception as e:
         traceback.print_exc()
         print('[' + timestamp + '] ' + str(e))
 
 
+def update_versioned_links(d):
+  for fn in os.listdir(d):
+    if fn.endswith('.html'):
+      changed = False
+      html = []
+ 
+      with open(d + '/' + fn, 'rt') as fh:
+        for ln in fh:
+          m = re.search(r'(?:href|src)="/([a-f0-9]{8})/(.+?)"', ln)
+          if m:
+            with open(d + '/' + m.group(2), 'rb') as fh2:
+              h = hashlib.sha256(fh2.read()).hexdigest()[:8]
+ 
+              if h != m.group(1):
+                ln = re.sub(m.group(1), h, ln)
+                changed = True
+ 
+          html.append(ln)
+ 
+      if changed:
+        with open(d + '/' + fn, 'wt') as fh:
+          fh.writelines(html)
+
+
 def w_directory(d):
   if not os.path.isdir(d):
     raise argparse.ArgumentTypeError("repository directory '" + d + "' must exist")
